@@ -2,6 +2,7 @@
 TO DO:
 
 Create a tool for feature analysis:
+|--remove obsolete imports
 |--plot hist to view distribution
 |--exp domain description
 |--Create a tool "outlier detection" -> in progress
@@ -45,14 +46,9 @@ import statsmodels.api as sm
 from statsmodels.stats import outliers_influence
 
 import inspect
-"""
-#for print_in_file()
-from docx import Document
-from docx.shared import Inches
-"""
+
 import matplotlib.pyplot as plt
-import plotly.figure_factory as ff
-import plotly.express as px
+
 import pandas as pd
 import numpy as np
 
@@ -61,36 +57,6 @@ from SALib.analyze import sobol
 
 import time
 
-"""
-Functions
-"""
-def plot_surface(a,b,c, **kwargs):
-    """
-    plot_surface: plot response surface in function of features a and b
-    
-    params:
-    a: 1D array (n,)
-    b: 1D array (n,)
-    c: 1D array (n,)
-    kwargs
-    
-    return: None
-    """
-    if hasattr(kwargs, "cmap"):
-        cmap=kwargs['cmap']
-    else:
-        cmap='Viridis'
-    
-    fig = plt.figure(figsize=(40,40))
-    ax = plt.axes(projection='3d')
-    Cmap = plt.get_cmap(cmap)  
-    surf = ax.plot_trisurf(a.ravel(), b.ravel(), c, cmap=Cmap, antialiased=True, edgecolor='none')     
-    fig.colorbar(surf, ax =ax, shrink=0.5, aspect=5)
-    ax.set_xlabel(f'{a.name}')
-    ax.set_ylabel(f'{b.name}')
-    ax.set_zlabel(f'{c.name}')
-    ax.set_title(f'Interaction: {c.name}')
-    fig.show()
 
 #interaction_dict will store all the data concerning the use of interactions
 interaction_dict={}
@@ -168,9 +134,9 @@ class Interaction:
         return None
         """
         if x is None:
-            x= np.linspace(-1, 1, 100)
+            x = np.linspace(-1, 1, 100)
         if y is None:
-            y= np.linspace(-1, 1, 100)
+            y = np.linspace(-1, 1, 100)
             
         x, y = np.meshgrid(x, y)
         x=pd.DataFrame(x.ravel(), columns=["x"])
@@ -506,16 +472,21 @@ class LBM_Regression:
             name = y.name
         else :
             name=y.columns
+
+        #mat =np.corrcoef(pd.concat([X, y], axis=1).T)
+        mat = np.ma.corrcoef(pd.concat([X, y], axis=1).T)
+        cols = X.columns.tolist() + [name]
         
-        self.corr_X = pd.DataFrame(np.corrcoef(pd.concat([X, y], axis=1).T), columns= X.columns.tolist() + [name])
-        return
+        self.corr_X = pd.DataFrame(mat, columns= cols)
+        #return
         
     def __feature_selection(self, res, corr_X, M, res_list, threshold):
     
         #identifier la meilleure variable explicative
         corr_X= np.abs(corr_X)
         
-        best_interaction = corr_X.iloc[-1, : -1].idxmax(axis=1)
+        #best_interaction = corr_X.iloc[-1, : -1].idxmax(axis=1)
+        best_interaction = corr_X.iloc[-1, : -1].idxmax()
         best_corr = M.loc[:, best_interaction]
         best_coef = corr_X.iloc[-1, : -1].max()
 
@@ -754,24 +725,32 @@ class LBM_Regression:
             self.model[i]['selected_features'] = []
             self.model[i]['results'] = y[i]
             self.model[i]['metrics'] = []
-            
+
             self.__compute_correlation_matrix(self.rescaled_features,  y[i])
             
             for reg in range(max_regressors_nb):
-                #identification de la meilleure interaction
+                #identification of the best interaction
                 self.model[i]['results'], self.model[i]['selected_features'] = self.__feature_selection(self.model[i]['results'], self.corr_X, self.rescaled_features, self.model[i]['selected_features'], threshold)
                 self.corr_X = self.__partial_correlations(self.corr_X, self.model[i]['selected_features'][-1][0])
 
                 self.model[i]['metrics'].append(self.__model_evaluation(self.model[i]['results']))
                 
             self.model[i]['nb_predictor'] = self.model[i]['metrics'].index(max(self.model[i]['metrics']))+1
-            self.model[i]['model_final'] = LinearRegression()
-                
+            #self.model[i]['model_final'] = LinearRegression()
+
+            #adding a column of bias to the selected features 
             data = pd.concat((self.model[i]['results'].iloc[:,1:self.model[i]['nb_predictor']+1], pd.DataFrame(np.ones(y[i].shape), columns=['intercept'])), axis=1)
-            model = sm.OLS(y[i], data)
-            self.model[i]['model_final'] = model.fit()
-            print(self.model[i]['model_final'].summary())
             
+            #cast y and data to numpy array
+            y_array = np.asarray(y[i], dtype = np.float64)
+            data_array = np.asarray(data, dtype= np.float64)
+            
+            #ordinary least square on transform dataset and target y[i]
+            #saving data into model dict
+            self.model[i]['model_final'] = sm.OLS(y_array, data_array).fit()
+            #print(self.model[i]['model_final'].summary())
+            print(self.model[i]['model_final'].summary())
+
             self.model[i]['y_pred'] = self.model[i]['model_final'].predict(data)
             
             
@@ -1015,51 +994,6 @@ class LBM_Regression:
             #plt.suptitle('Overview of the modelisation')
         return
 
-    """
-    def print_in_file(self, title: str ='title'):
-                
-
-        document = Document()
-
-        document.add_heading(f'{title}', 0)
-        if self.with_fit:
-            document.add_heading('Modelization', level=1)
-            document.add_heading('Model', level=2, style='List Number')
-            document.add_heading('Metrics', level=2, style='List Number')
-            document.add_heading('Plots', level=2, style='List Number')
-            document.add_heading('Sensibility analysis', level=2, style='List Number')
-    
-            records = (
-            (3, '101', 'Spam'),
-            (7, '422', 'Eggs'),
-            (4, '631', 'Spam, spam, eggs, and spam')
-            )
-
-            table = document.add_table(rows=1, cols=3)
-            hdr_cells = table.rows[0].cells
-            hdr_cells[0].text = 'Qty'
-            hdr_cells[1].text = 'Id'
-            hdr_cells[2].text = 'Desc'
-            for qty, id, desc in records:
-                row_cells = table.add_row().cells
-                row_cells[0].text = str(qty)
-                row_cells[1].text = id
-                row_cells[2].text = desc
-            
-            document.add_page_break()
-
-        if self.with_optimization:
-            document.add_heading('Optimization', level=1)
-            document.add_heading('Targets', level=2, style='List Number')
-            document.add_heading('Best Trials', level=2, style='List Number')
-            document.add_heading('Pareto', level=2, style='List Number')
-        document.add_picture('monty-truth.png', width=Inches(1.25))
-        document.save(f'{title}.docx')
-
-    def print_in_file(self):
-        return #fichier avec données enregistrées et formatées
-    """
-
     def __extract_features(self, experimental_domain: dict):
         screened_var=[]
         set_mix_var =[]
@@ -1256,121 +1190,3 @@ class LBM_Regression:
                 plt.show()
 
         return Sobol_list
-
-class OutliersInspection:
-    def __init__(self, other:object):
-        self.other=other
-        self.frame_list=[]
-        for i in self.other.y:
-            self.outliers = outliers_influence.OLSInfluence(self.other.model[i]['model_final'])
-            self.frame_list.append(self.outliers.summary_frame())
-        return self
-        
-    def cooks_distance(self, plot: bool =True):
-        for i in self.other.y:
-            threshold = 4/self.outliers.summary_frame().shape[0]
-            
-            print(f'threshold (4/n) = {round(threshold,3)}' )
-            outliers_list= []
-            for n in range(0,self.outliers.summary_frame().shape[0]):
-                if self.outliers.summary_frame().at[n, 'cooks_d'] >= threshold:
-                    outliers_list.append((n, self.outliers.summary_frame().at[n, 'cooks_d']))
-            print(f'potential outliers : {outliers_list}')
-            if plot:
-                #plt.figure()
-                plt.scatter(range(0,self.outliers.summary_frame().shape[0]), self.outliers.summary_frame()['cooks_d'], label=i)
-            print(self.outliers.summary_table())
-            
-        plt.plot([0, self.outliers.summary_frame().shape[0]], [threshold, threshold], c='r', label='threshold')
-        plt.xlabel('Observation indices')
-        plt.ylabel('Cook\'s distance')
-        plt.legend(bbox_to_anchor=(1.05, 0.85), loc='upper left', borderaxespad=0.)
-        plt.show()
-        
-        return self.frame_list
-    
-    def mahalanobis_distance(self, plot:bool=True):
-      #To Do
-        """
-        mahalanobis_distance:
-        D**2 = (x-µ)**T.C**(-1).(x-µ)
-        where, 
-        - D**2        is the square of the Mahalanobis distance. 
-        - x          is the vector of the observation (row in a dataset), 
-        - µ          is the vector of mean values of independent variables (mean of each column), 
-        - C**(-1)     is the inverse covariance matrix of independent variables.
-        """
-        for i in self.orner.y:
-            diff_x_u = self.other.X - np.mean(self.other.X, axis=0)
-            if not cov:
-                cov = np.cov(self.other.X.values.T)
-            inv_covmat = sp.linalg.inv(cov)
-            left_term = np.dot(diff_x_u, inv_covmat)
-            self.mahal_d = np.dot(left_term, diff_x_u.T)
-        #ajouter Mahalanobis a outlier summary
-        
-        if plot:
-          print("Not yet implemented")
-          plt.scatter(range(0,self.outliers.summary_frame().shape[0]), self.mahal_d.diagonal(), label=i)
-        
-        return self.mahal_d.diagonal()
-
-    def z_score(self, ddof:int=0, plot:bool=True)-> object:
-        """
-        https://medium.com/clarusway/z-score-and-how-its-used-to-determine-an-outlier-642110f3b482
-        
-        z_score: calculate the z-score of the targets.
-            z-score = (y - mean(y)) / std(y)
-        z-score is the number of standard deviations away from the mean the data point is.
-        
-        params:
-            ddof: degre of freedom
-        returns:
-            DataFrame
-        """
-        df = pd.DataFrame()
-        for col in self.other.y.columns:
-            col_zscore = col + "_zscore"
-            df[col_zscore] = (self.other.y[col] - self.other.y[col].mean())/self.other.y[col].std(ddof=ddof)
-        """
-        df["outlier"] = (abs(df[col + "_zscore"])>3).astype(int)
-        print("number of outliers = " + str(df.outlier.value_counts()[1]))
-        """
-        return df
-
-"""
-    def z_score(df, col, min_z=1, max_z = 5, step = 0.1, print_list = False):
-        z_scores = df["Data_zscore"]
-        threshold_list = []
-        for threshold in np.arange(min_z, max_z, step):
-            threshold_list.append((threshold, len(np.where(z_scores > threshold)[0])))
-            df_outlier = pd.DataFrame(threshold_list, columns = ['threshold', 'outlier_count'])
-            df_outlier['pct'] = (df_outlier.outlier_count - df_outlier.outlier_count.shift(-1))/df_outlier.outlier_count*100
-        plt.plot(df_outlier.threshold, df_outlier.outlier_count)
-        best_treshold = round(df_outlier.iloc[df_outlier.pct.argmax(), 0],2)
-        outlier_limit = int(df[col].dropna().mean() + (df[col].dropna().std()) * df_outlier.iloc[df_outlier.pct.argmax(), 0])
-        percentile_threshold = stats.percentileofscore(df[col].dropna(), outlier_limit)
-        plt.vlines(best_treshold, 0, df_outlier.outlier_count.max(), 
-               colors="r", ls = ":"
-              )
-        plt.annotate("Zscore : {}\nValue : {}\nPercentile : {}".format(best_treshold, outlier_limit,(np.round(percentile_threshold, 3),np.round(100-percentile_threshold, 3))),(best_treshold, df_outlier.outlier_count.max()/2))
-        #plt.show()
-        if print_list:
-            print(df_outlier)
-        return (plt, df_outlier, best_treshold, outlier_limit, percentile_threshold)
-"""
-"""
-    def outlier_inspect(df, col, min_z=1, max_z = 5, step = 0.2, max_hist = None, bins = 50):
-        fig = plt.figure(figsize=(20, 6))
-        fig.suptitle(col, fontsize=16)
-        plt.subplot(1,3,1)
-        if max_hist == None:
-            sns.histplot(df[col], kde=False, bins = 50,color="r")
-        else :
-            sns.distplot(df[df[col]<=max_hist][col], kde=False, bins = 50)
-        plt.subplot(1,3,2)
-        sns.boxplot(df[col])
-        plt.subplot(1,3,3)
-        z_score_inspect = z_score(df, col, min_z=min_z, max_z = max_z, step = step)
-        plt.show()
-"""
\ No newline at end of file
